-- Puffin: A Tojaso FowlPlay Project
-- Model component for world map, inspired by ElvUI map module

local _, Puffin = ...
local EF = Puffin.Error
local DB = Puffin.Database

local Model_Map = {}

-- Initialize by adding player and mouse coordinates to the map frame
function Model_Map.Configure()
	local f = CreateFrame("Frame", "Puffin_Coordinates", WorldMapFrame)
	f:SetFrameLevel(WorldMapDetailFrame:GetFrameLevel() + 1)
	f:SetFrameStrata(WorldMapDetailFrame:GetFrameStrata())
	f._player = f:CreateFontString(nil, "OVERLAY")
	f._mouse = f:CreateFontString(nil, "OVERLAY")
	local p, m = f._player, f._mouse
	local font = DB.GetAttribute("defaults", "media", "font")
	local fsize = DB.GetAttribute("defaults", "media", "fontSize")
	local c = DB.GetAttribute("defaults", "media", "fontColor")
	p:SetFont(font, fsize, "None"); m:SetFont(font, fsize, "None")
	p:SetShadowColor(0, 0, 0, 1); p:SetShadowOffset(1, -1); m:SetShadowColor(0, 0, 0, 1); m:SetShadowOffset(1, -1)
	p:SetTextColor(c.r, c.g, c.b); m:SetTextColor(c.r, c.g, c.b)
	p:SetJustifyH("LEFT"); m:SetJustifyH("RIGHT")
	p:SetJustifyV("BOTTOM"); m:SetJustifyV("BOTTOM")
	p:SetSize(160, fsize + 8); m:SetSize(160, fsize + 8)
	p:ClearAllPoints(); p:SetPoint("BOTTOMLEFT", WorldMapDetailFrame, "BOTTOMLEFT", 40, 5)
	m:ClearAllPoints(); m:SetPoint("BOTTOMRIGHT", WorldMapDetailFrame, "BOTTOMRIGHT", -40, 5)
	
	SetCVar("mapAnimMinAlpha", 0.5) -- minimum alpha when moving
	SetCVar("mapFade", 1) -- fade when moving
end

-- Update coordinates when map is visible
function Model_Map.OnUpdate()
	if WorldMapDetailFrame:IsVisible() then -- check if map is actually visible
		local pset, mset = false, false
		local p = Puffin_Coordinates._player
		local m = Puffin_Coordinates._mouse
		local x, y = GetPlayerMapPosition("player") -- could be in a restricted area
		if x and y then
			x = math.floor(1000 * x + 0.5) / 10
			y = math.floor(1000 * y + 0.5) / 10
			if x > 0 and y > 0 then p:SetText("Player: "..x..", "..y) end
			p:Show(); pset = true
		end
	
		local wf = WorldMapDetailFrame
		if wf:IsMouseOver() then
			local scale = wf:GetEffectiveScale()
			local w = wf:GetWidth()
			local h = wf:GetHeight()
			local wx, wy = wf:GetCenter()
			cx, cy = GetCursorPosition()
			x = (cx / scale - (wx - (w / 2))) / w
			y = (wy + (h / 2) - cy / scale) / h
			if x >= 0 and x <= 1 and y >= 0 and y <= 1 then -- range check
				x = math.floor(1000 * x + 0.5) / 10
				y = math.floor(1000 * y + 0.5) / 10
				if x > 0 and y > 0 then m:SetText("Cursor: "..x..", "..y) end			
				m:Show(); mset = true
			end
		end
		if not pset then p:Hide() end
		if not mset then m:Hide() end
	end
end

Puffin.RegisterCallback("Puffin_Configure", Model_Map.Configure)
Puffin.RegisterCallback("Puffin_Update", Model_Map.OnUpdate)
