-- Puffin: A Tojaso FowlPlay Project
-- Model component for setting up minimap

local _, Puffin = ...
local EF = Puffin.Error
local DB = Puffin.Database

local Model_Minimap = {}

local hideEvent = function(frame) frame:Hide() end -- handler for events that might show a hidden frame
local ignoreEvent = function() end -- handler for events that want to ignore

local requiredButtons = { -- these are required and must be handled by specific minimap code
	GameTimeFrame = "Calendar",
	MiniMapTracking = "Tracking",
	MinimapZoneTextButton = "Zone",
	MinimapZoomIn = "Zoom In",
	MinimapZoomOut = "Zoom Out",
	MiniMapWorldMapButton = "Map",
	TimeManagerClockButton = "Clock",
	GuildInstanceDifficulty = "Guild Instance",
	MiniMapChallengeMode = "Challenge",
	MiniMapInstanceDifficulty = "Instance",
	MiniMapMailFrame = "Mail",
	MiniMapRecordingButton = "Video",
	MiniMapVoiceChatFrame = "Voice",
	QueueStatusMinimapButton = "Queue",
	GarrisonLandingPageMinimapButton = "Garrison",
}

-- Look in a list of frames for minimap buttons to hide
local function CheckFrames(...)
	for i = 1, select("#", ...) do
		local f = select(i, ...)
		local n = f:GetName()
		if n then
			-- Puffin.Debug("frame:", n)
			if not requiredButtons[f] then f:SetParent(Model_Minimap.hiddenFrame) end
		end
	end
end

-- Hide all minimap buttons except for those in required list
local function HideButtons()
	-- Puffin.Debug("HideButtons")
	CheckFrames(Minimap:GetChildren()) -- search for buttons among the frames anchored to the minimap
end

-- Toggle the tracking menu
local function ShowTrackingMenu(_, button)
	if button == "RightButton" then
		ToggleDropDownMenu(1, nil, MiniMapTrackingDropDown, "cursor")
	end
end

-- Initialize by taking control of the minimap and registering for relevant events
function Model_Minimap.OnInitialize()
	local Minimap = Minimap
	Minimap:SetParent(UIParent)
	MinimapCluster:EnableMouse(false)
	Minimap:ClearAllPoints()
	Minimap:SetMaskTexture("Interface\\BUTTONS\\WHITE8X8") -- makes it a square
	MinimapNorthTag.Show = MinimapNorthTag.Hide
	MinimapNorthTag:Hide()
	MinimapBorder:Hide()
	MinimapBorderTop:Hide()
	Minimap:SetArchBlobRingScalar(0)
	Minimap:SetArchBlobRingAlpha(0)
	Minimap:SetQuestBlobRingScalar(0)
	Minimap:SetQuestBlobRingAlpha(0)
	MinimapZoomIn:Hide()
	MinimapZoomOut:Hide()
	MiniMapVoiceChatFrame:SetScript("OnShow", hideFrame)
	MiniMapVoiceChatFrame:Hide()
	MiniMapVoiceChatFrame:UnregisterAllEvents()
	TimeManagerClockButton:Hide()
	TimeManagerClockButton.bmShow = TimeManagerClockButton.Show
	TimeManagerClockButton.Show = noop
	MiniMapWorldMapButton:SetScript("OnShow", hideEvent)
	MiniMapWorldMapButton:Hide()
	MiniMapWorldMapButton:UnregisterAllEvents()
	-- MinimapZoneTextButton:Hide()
	-- MinimapZoneTextButton.bmShow = MinimapZoneTextButton.Show
	-- MinimapZoneTextButton.Show = ignoreEvent
	MinimapZoneTextButton:ClearAllPoints()
	MinimapZoneTextButton:SetPoint("TOP", Minimap, "BOTTOM", 0, -22)
	GarrisonLandingPageMinimapButton:Hide()
	GarrisonLandingPageMinimapButton.bmShow = GarrisonLandingPageMinimapButton.Show
	GarrisonLandingPageMinimapButton.Show = ignoreEvent
	MiniMapInstanceDifficulty:SetScript("OnShow", hideEvent)
	MiniMapInstanceDifficulty:Hide()
	GuildInstanceDifficulty:SetScript("OnShow", hideEvent)
	GuildInstanceDifficulty:Hide()
	QueueStatusMinimapButton:ClearAllPoints()
	QueueStatusMinimapButton:SetParent(Minimap)
	QueueStatusMinimapButton:SetPoint("BOTTOMLEFT", Minimap, "BOTTOMLEFT", -10, -10)	
	GameTimeFrame:Hide()
	
	Minimap:EnableMouseWheel(true)
	Minimap:SetScript("OnMouseWheel", function(m, x) if x > 0 then MinimapZoomIn:Click() elseif x < 0 then MinimapZoomOut:Click() end end)
	
	if Minimap:GetScript("OnMouseDown") then
		Minimap:HookScript("OnMouseDown", ShowTrackingMenu)
	else
		Minimap:SetScript("OnMouseDown", ShowTrackingMenu)
	end
	
	Model_Minimap.hiddenFrame = CreateFrame("Frame") -- this frame is used to reparent buttons we want to hide
	Model_Minimap.hiddenFrame:Hide()
	
	Puffin.RegisterCallback("Puffin_Hide_Minimap_Buttons", HideButtons)
	Puffin.StartTimer("Puffin_Hide_Minimap_Buttons", 2) -- delay to give addons a chance to register buttons
end

-- Hide minimap during pet battles
local function PetBattleStart() Minimap:Hide() end
local function PetBattleEnd() Minimap:Show() end

Puffin.RegisterCallback("Puffin_Initialize_Priority", Model_Minimap.OnInitialize)
Puffin.RegisterCallback("PET_BATTLE_OPENING_START", PetBattleStart)
Puffin.RegisterCallback("PET_BATTLE_CLOSE", PetBattleEnd)
