-- Puffin: A Tojaso FowlPlay Project
-- Model component for customized nameplates, inspired by Zork and others

local _, Puffin = ...
local EF = Puffin.Error
local DB = Puffin.Database

local Model_Nameplates = {}

local count = 0 -- number of nameplates
local hideEvent = function(frame) frame:Hide() end -- handler for events that might show a hidden frame

-- When a nameplate is created, hide the blizzard frame and create a view to display in its place
function Model_Nameplates:Nameplate_Created(nameplate)
	if KuiNameplates then return end
	-- Puffin.Debug("NAME_PLATE_CREATED", nameplate:GetName(), nameplate:GetSize())
	
	local c = Puffin.Classes.View_Nameplate
	if not c then return end
	local name = nameplate:GetName()
	local v = { enabled = true, class = "View_Nameplate", layout = "nameplates", strata = "BACKGROUND", nameplate = nameplate,
		point = "CENTER", anchor = "CENTER", offsetX = 0, offsetY = 0, }
	local view = c:new(name, v)
	assert(view, EF("Model_Nameplates", "nameplate not created " .. name))
	nameplate.puffin_frame = view
	count = count + 1
    if nameplate.UnitFrame then nameplate.UnitFrame:Hide(); nameplate.UnitFrame:HookScript('OnShow', hideEvent) end
end

-- Called when a nameplate is added for a particular unit
function Model_Nameplates:Nameplate_Added(unit)
	if KuiNameplates then return end
	local nameplate = C_NamePlate.GetNamePlateForUnit(unit)
	-- Puffin.Debug("NAME_PLATE_UNIT_ADDED", unit, nameplate.puffin_frame.name)
	nameplate.puffin_frame.unit = unit
	local level = string.gsub(unit, "nameplate", "")
	nameplate.puffin_frame.level = (tonumber(level) or 1) * 10
end

-- Called when a nameplate is removed for a particular unit 
function Model_Nameplates:Nameplate_Removed(unit)
	if KuiNameplates then return end
	local nameplate = C_NamePlate.GetNamePlateForUnit(unit)
	-- Puffin.Debug("NAME_PLATE_UNIT_REMOVED", unit, nameplate.puffin_frame.name)
	nameplate.puffin_frame.unit = false
end
 
Puffin.RegisterCallback("NAME_PLATE_CREATED", Model_Nameplates.Nameplate_Created)
Puffin.RegisterCallback("NAME_PLATE_UNIT_ADDED", Model_Nameplates.Nameplate_Added)
Puffin.RegisterCallback("NAME_PLATE_UNIT_REMOVED", Model_Nameplates.Nameplate_Removed)
