-- Puffin: A Tojaso FowlPlay Project
-- Model component for secure unit frames, including hiding Blizzard frames

local _, Puffin = ...
local EF = Puffin.Error
local DB = Puffin.Database

local Model_Units = {}

local function DisableFrame(f) if f then f:UnregisterAllEvents(); f:SetScript("OnUpdate", nil); f:Hide(); f:SetAlpha(0) end end

local function DisableUnitFrame(f)
	if f then
		f:SetAlpha(0)
		f:EnableMouse(false)
		f:ClearAllPoints()
		f:SetPoint("TOPLEFT", UIParent, "BOTTOMLEFT", -400, -400) -- move offscreen
		local name = f:GetName()
		DisableFrame(f)
		if name then DisableFrame(_G[name.."HealthBar"]); DisableFrame(_G[name.."ManaBar"]); DisableFrame(_G[name.."SpellBar"]) end
	end
end

-- Show tooltip when entering a unit frame
local function Unit_OnEnter(b)
	if GetCVar("UberTooltips") == "1" then
		GameTooltip_SetDefaultAnchor(GameTooltip, b)
	else
		GameTooltip:SetOwner(b, "BOTTOMRIGHT")
	end
	local unit = b.unit
	if unit == "player" or unit == "pet" then
		if UnitHasVehicleUI("player") then -- switch units around if player in vehicle
			if unit == "player" then unit = "pet" else unit = "player" end
		end
	end
	GameTooltip:SetUnit(unit)
end

-- Hide tooltip when leaving a unit frame
local function Unit_OnLeave() GameTooltip:Hide() end

-- Hide blizzard frames, based on implementation in Stuf
function Model_Units.OnInitialize()
	-- hide blizzard frames, based on implementation in Stuf
	DisableUnitFrame(PlayerFrame)
	DisableUnitFrame(TargetFrame)
	DisableUnitFrame(TargetofTargetFrame)
	DisableUnitFrame(ComboFrame)
	DisableUnitFrame(PetFrame)
	DisableUnitFrame(FocusFrame)
	DisableUnitFrame(TargetofFocusFrame)
	DisableUnitFrame(Boss1TargetFrame)
	DisableUnitFrame(Boss2TargetFrame)
	DisableUnitFrame(Boss3TargetFrame)
	DisableUnitFrame(Boss4TargetFrame)
	DisableUnitFrame(Boss5TargetFrame)
end

-- Initialize unit frames based on configuration, this happens before client can be in combat
function Model_Units.Configure()	
	local units = DB.GetRecord("units", "default")
	assert(units, EF("Model_Units", "units not found"))
	for _, unit in pairs(units) do -- create secure frames for all units in configuration
		local dt = DB.GetRecord("units", unit)
		if dt and dt.enabled then
			local Pixel = Puffin.Pixel
			assert(type(Pixel) == "function", EF("Model_Units", "pixel function missing"))
			local b = CreateFrame("Button", dt.frame, UIParent, "SecureUnitButtonTemplate")
			b:SetSize(Pixel(dt.width), Pixel(dt.height)) -- set size of button
			b:SetPoint(dt.point, nil, dt.relative, Pixel(dt.offsetX), Pixel(dt.offsetY)) -- set position of button (no need to align since invisible)
			b.unit = unit -- for tooltips
			b:SetAttribute("unit", unit)
			-- if unit == "player" or unit == "pet" then b:SetAttribute("toggleForVehicle", true) end
			b:SetAttribute("toggleForVehicle", true) -- probably need to set this for all units
			RegisterUnitWatch(b)
			b:SetScript("OnEnter", Unit_OnEnter)
			b:SetScript("OnLeave", Unit_OnLeave)
			b:EnableMouse(true)
			b:RegisterForClicks("AnyUp")
			b:SetAttribute("type1", "target") -- left click target
			b:SetAttribute("type2", "togglemenu") -- right click menu
			b:Enable()
		end
	end
end

Puffin.RegisterCallback("Puffin_Initialize_Priority", Model_Units.OnInitialize)
Puffin.RegisterCallback("Puffin_Configure_Priority", Model_Units.Configure)
